@@ -118,8 +118,8 @@ def open(path, convert=False, shuffle=False, copy_index=True, *args, **kwargs):
 
     Example:
 
-    >>> ds = vaex.open('sometable.hdf5')
-    >>> ds = vaex.open('somedata*.csv', convert='bigdata.hdf5')
+    >>> df = vaex.open('sometable.hdf5')
+    >>> df = vaex.open('somedata*.csv', convert='bigdata.hdf5')
 
     :param str or list path: local or absolute path to file, or glob string, or list of paths
     :param convert: convert files to an hdf5 file for optimization, can also be a path
@@ -130,6 +130,24 @@ def open(path, convert=False, shuffle=False, copy_index=True, *args, **kwargs):
     :return: return a DataFrame on succes, otherwise None
     :rtype: DataFrame
 
+    S3 support:
+
+    Vaex supports streaming in hdf5 files from Amazon AWS object storage S3.
+    Files are by default cached in $HOME/.vaex/file-cache/s3 such that successive access
+    it as fast as native disk access. The following url parameters control S3 options:
+
+     * anon: Use anonymous access or not (false by default). (Allowed values are: true,True,1,false,False,0)
+     * use_cache: Use the disk cache or not, only set to false if the data should be accessed once. (Allowed values are: true,True,1,false,False,0)
+     * profile_name and other arguments are passed to :py:class:`s3fs.core.S3FileSystem`
+
+    All arguments can also be passed as kwargs, but then arguments such as `anon` can only be a boolean, not a string.
+
+    Examples:
+
+    >>> df = vaex.open('s3://vaex/taxi/yellow_taxi_2015_f32s.hdf5?anon=true')
+    >>> df = vaex.open('s3://vaex/taxi/yellow_taxi_2015_f32s.hdf5', anon=True)  # Note that anon is a boolean, not the string 'true'
+    >>> df = vaex.open('s3://mybucket/path/to/file.hdf5?profile_name=myprofile')
+
     """
     import vaex
     try:
@@ -374,10 +392,14 @@ def from_pandas(df, name="pandas", copy_index=True, index_name="index"):
     :rtype: DataFrame
     """
     import six
+    import pandas as pd
+    import numpy as np
     vaex_df = vaex.dataframe.DataFrameArrays(name)
 
     def add(name, column):
         values = column.values
+        if isinstance(values, pd.core.arrays.integer.IntegerArray):
+            values = np.ma.array(values._data, mask=values._mask)
         try:
             vaex_df.add_column(name, values)
         except Exception as e:
@@ -635,13 +657,63 @@ def register_dataframe_accessor(name, cls=None, override=False):
 
 
 for entry in pkg_resources.iter_entry_points(group='vaex.namespace'):
-    logger.debug('adding vaex namespace: ' + entry.name)
+    logger.warning('(DEPRECATED, use vaex.dataframe.accessor) adding vaex namespace: ' + entry.name)
     try:
         add_namespace = entry.load()
         add_namespace()
     except Exception:
         logger.exception('issue loading ' + entry.name)
 
+_df_lazy_accessors = {}
+
+
+class _lazy_accessor(object):
+    def __init__(self, name, scope, loader):
+        """When adding an accessor geo.cone, scope=='geo', name='cone', scope may be falsy"""
+        self.loader = loader
+        self.name = name
+        self.scope = scope
+
+    def __call__(self, obj):
+        if self.name in obj.__dict__:
+            return obj.__dict__[self.name]
+        else:
+            cls = self.loader()
+            accessor = cls(obj)
+            obj.__dict__[self.name] = accessor
+            fullname = self.name
+            if self.scope:
+                fullname = self.scope + '.' + self.name
+            if fullname in _df_lazy_accessors:
+                for name, scope, loader in _df_lazy_accessors[fullname]:
+                    assert fullname == scope
+                    setattr(cls, name, property(_lazy_accessor(name, scope, loader)))
+        return obj.__dict__[self.name]
+
+
+def _add_lazy_accessor(name, loader, target_class=vaex.dataframe.DataFrame):
+    """Internal use see tests/internal/accessor_test.py for usage
+
+    This enables us to have df.foo.bar accessors that lazily loads the modules.
+    """
+    parts = name.split('.')
+    target_class = vaex.dataframe.DataFrame
+    if len(parts) == 1:
+        setattr(target_class, parts[0], property(_lazy_accessor(name, None, loader)))
+    else:
+        scope = ".".join(parts[:-1])
+        if scope not in _df_lazy_accessors:
+            _df_lazy_accessors[scope] = []
+        _df_lazy_accessors[scope].append((parts[-1], scope, loader))
+
+
+for entry in pkg_resources.iter_entry_points(group='vaex.dataframe.accessor'):
+    logger.debug('adding vaex accessor: ' + entry.name)
+    def loader(entry=entry):
+        return entry.load()
+    _add_lazy_accessor(entry.name, loader)
+
+
 for entry in pkg_resources.iter_entry_points(group='vaex.plugin'):
     logger.debug('adding vaex plugin: ' + entry.name)
     try:
@@ -650,7 +722,6 @@ for entry in pkg_resources.iter_entry_points(group='vaex.plugin'):
     except Exception:
         logger.exception('issue loading ' + entry.name)
 
-from . import geo
 
 def concat(dfs):
     '''Concatenate a list of DataFrames.
